(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('prosemirror-state'), require('prosemirror-view'), require('prosemirror-commands'), require('ngx-editor/helpers'), require('ngx-editor/commands')) :
    typeof define === 'function' && define.amd ? define('ngx-editor/plugins', ['exports', 'prosemirror-state', 'prosemirror-view', 'prosemirror-commands', 'ngx-editor/helpers', 'ngx-editor/commands'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global['ngx-editor'] = global['ngx-editor'] || {}, global['ngx-editor'].plugins = {}), global.prosemirrorState, global.prosemirrorView, global.prosemirrorCommands, global['ngx-editor'].helpers, global['ngx-editor'].commands));
}(this, (function (exports, prosemirrorState, prosemirrorView, prosemirrorCommands, helpers, commands) { 'use strict';

    var DEFAULT_PLACEHOLDER = 'Type Here...';
    var PLACEHOLDER_CLASSNAME = 'NgxEditor__Placeholder';
    function placeholderPlugin(text) {
        if (text === void 0) { text = DEFAULT_PLACEHOLDER; }
        return new prosemirrorState.Plugin({
            key: new prosemirrorState.PluginKey('placeholder'),
            props: {
                decorations: function (state) {
                    var _a;
                    var doc = state.doc;
                    if (doc.childCount === 1 && ((_a = doc === null || doc === void 0 ? void 0 : doc.firstChild) === null || _a === void 0 ? void 0 : _a.isTextblock) && doc.firstChild.content.size === 0) {
                        var placeHolderEl = document.createElement('span');
                        placeHolderEl.classList.add(PLACEHOLDER_CLASSNAME);
                        placeHolderEl.textContent = text;
                        return prosemirrorView.DecorationSet.create(doc, [prosemirrorView.Decoration.widget(1, placeHolderEl)]);
                    }
                    return prosemirrorView.DecorationSet.empty;
                }
            }
        });
    }

    /* tslint:disable */
    var bold = "\n  <path d=\"M15.6 10.79c.97-.67 1.65-1.77 1.65-2.79 0-2.26-1.75-4-4-4H7v14h7.04c2.09 0 3.71-1.7 3.71-3.79 0-1.52-.86-2.82-2.15-3.42zM10 6.5h3c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5h-3v-3zm3.5 9H10v-3h3.5c.83 0 1.5.67 1.5 1.5s-.67 1.5-1.5 1.5z\" />\n  <path d=\"M0 0h24v24H0z\" fill=\"none\" />\n";

    var italic = "\n  <path d=\"M0 0h24v24H0z\" fill=\"none\" />\n  <path d=\"M10 4v3h2.21l-3.42 8H6v3h8v-3h-2.21l3.42-8H18V4z\" />\n";

    var code = "\n<path d=\"M0 0h24v24H0V0z\" fill=\"none\"/>\n<path d=\"M9.4 16.6L4.8 12l4.6-4.6L8 6l-6 6 6 6 1.4-1.4zm5.2 0l4.6-4.6-4.6-4.6L16 6l6 6-6 6-1.4-1.4z\"/>\n";

    var orderedList = "\n<path d=\"M2 17h2v.5H3v1h1v.5H2v1h3v-4H2v1zm1-9h1V4H2v1h1v3zm-1 3h1.8L2 13.1v.9h3v-1H3.2L5 10.9V10H2v1zm5-6v2h14V5H7zm0 14h14v-2H7v2zm0-6h14v-2H7v2z\"/>\n<path d=\"M0 0h24v24H0z\" fill=\"none\"/>\n";

    var bulletList = "\n<path d=\"M0 0h24v24H0V0z\" fill=\"none\"/>\n<path d=\"M4 10.5c-.83 0-1.5.67-1.5 1.5s.67 1.5 1.5 1.5 1.5-.67 1.5-1.5-.67-1.5-1.5-1.5zm0-6c-.83 0-1.5.67-1.5 1.5S3.17 7.5 4 7.5 5.5 6.83 5.5 6 4.83 4.5 4 4.5zm0 12c-.83 0-1.5.68-1.5 1.5s.68 1.5 1.5 1.5 1.5-.68 1.5-1.5-.67-1.5-1.5-1.5zM7 19h14v-2H7v2zm0-6h14v-2H7v2zm0-8v2h14V5H7z\"/>\n";

    var quote = "\n<path d=\"M0 0h24v24H0z\" fill=\"none\"/><path d=\"M6 17h3l2-4V7H5v6h3zm8 0h3l2-4V7h-6v6h3z\"/>\n";

    var link = "\n<path d=\"M0 0h24v24H0z\" fill=\"none\"/><path d=\"M3.9 12c0-1.71 1.39-3.1 3.1-3.1h4V7H7c-2.76 0-5 2.24-5 5s2.24 5 5 5h4v-1.9H7c-1.71 0-3.1-1.39-3.1-3.1zM8 13h8v-2H8v2zm9-6h-4v1.9h4c1.71 0 3.1 1.39 3.1 3.1s-1.39 3.1-3.1 3.1h-4V17h4c2.76 0 5-2.24 5-5s-2.24-5-5-5z\"/>\n";

    // Icons source: https://material.io/
    var DEFAULT_ICON_HEIGHT = 20;
    var DEFAULT_ICON_WIDTH = 20;
    var icons = {
        bold: bold,
        italic: italic,
        code: code,
        ordered_list: orderedList,
        bullet_list: bulletList,
        quote: quote,
        link: link
    };
    // Helper function to create menu icons
    function getIconSvg(name, width, height) {
        if (width === void 0) { width = DEFAULT_ICON_WIDTH; }
        if (height === void 0) { height = DEFAULT_ICON_HEIGHT; }
        var path = icons[name] || '<path></path>';
        return "\n  <svg\n    xmlns=\"http://www.w3.org/2000/svg\"\n    viewBox=\"0 0 24 24\"\n    fill=\"black\"\n    height=" + height + "\n    width=" + width + "\n  >\n    " + path + "\n  </svg>\n  ";
    }

    var flatDeep = function (arr, d) {
        if (d === void 0) { d = 1; }
        return d > 0
            ? arr.reduce(function (acc, val) { return acc.concat(Array.isArray(val) ? flatDeep(val, d - 1) : val); }, [])
            : arr.slice();
    };
    var ɵ0 = flatDeep;

    var menuItemsMeta = {
        bold: {
            key: 'strong',
            i18nKey: 'bold',
            icon: 'bold',
            type: 'mark',
        },
        italic: {
            key: 'em',
            i18nKey: 'italics',
            icon: 'italic',
            type: 'mark',
        },
        code: {
            key: 'code',
            i18nKey: 'code',
            icon: 'code',
            type: 'mark',
        },
        ordered_list: {
            key: 'ordered_list',
            i18nKey: 'ordered_list',
            icon: 'ordered_list',
            type: 'node',
        },
        bullet_list: {
            key: 'bullet_list',
            i18nKey: 'bullet_list',
            icon: 'bullet_list',
            type: 'node',
        },
        h1: {
            key: 'heading',
            i18nKey: 'heading',
            attrs: {
                level: 1,
            },
            type: 'node'
        },
        h2: {
            key: 'heading',
            i18nKey: 'heading',
            attrs: {
                level: 2,
            },
            type: 'node'
        },
        h3: {
            key: 'heading',
            i18nKey: 'heading',
            attrs: {
                level: 3,
            },
            type: 'node'
        },
        h4: {
            key: 'heading',
            i18nKey: 'heading',
            attrs: {
                level: 4,
            },
            type: 'node'
        },
        h5: {
            key: 'heading',
            i18nKey: 'heading',
            attrs: {
                level: 5,
            },
            type: 'node'
        },
        h6: {
            key: 'heading',
            i18nKey: 'heading',
            attrs: {
                level: 6,
            },
            type: 'node'
        },
        blockquote: {
            key: 'blockquote',
            i18nKey: 'blockquote',
            icon: 'quote',
            type: 'node'
        }
    };

    var SEPERATOR_CLASSNAME = 'NgxEditor__Seperator';
    var MENU_ITEM_CLASSNAME = 'NgxEditor__MenuItem';
    var ACTIVE_MENU_ITEM_CLASSNAME = MENU_ITEM_CLASSNAME + "--Active";
    var DISABLED_CLASSNAME = 'NgxEditor--Disabled';
    var DROPDWON_ITEM_CLASSNAME = 'NgxEditor__Dropdown';
    var DROPWDOWN_OPEN_CLASSNAME = DROPDWON_ITEM_CLASSNAME + "--Open";
    var ACTIVE_DROPDOWN_ITEM_CLASSNAME = DROPDWON_ITEM_CLASSNAME + "--Active";
    var SELECTED_DROPDOWN_ITEM_CLASSNAME = DROPDWON_ITEM_CLASSNAME + "--Selected";
    var DROPDOWN_ITEMS = new Map();
    DROPDOWN_ITEMS.set('heading', ['h1', 'h2', 'h3', 'h4', 'h5', 'h6']);
    var DropDownView = /** @class */ (function () {
        function DropDownView(dropdownGroup, dropdownFields, editorView, options) {
            this.updates = [];
            this.dropdownGroup = dropdownGroup;
            this.dropdownFields = dropdownFields;
            this.editorView = editorView;
            this.options = options;
        }
        DropDownView.prototype.getWrapperDom = function () {
            var _this = this;
            var isDropdownOpen = false;
            var dropdown = document.createElement('div');
            var labels = this.options.labels;
            dropdown.classList.add(DROPDWON_ITEM_CLASSNAME);
            var dropdownText = document.createElement('div');
            dropdownText.classList.add(DROPDWON_ITEM_CLASSNAME + "__Text");
            dropdownText.textContent = labels[this.dropdownGroup];
            dropdown.appendChild(dropdownText);
            // create dropdown list
            var dropdownMenu = document.createElement('div');
            dropdownMenu.classList.add(DROPDWON_ITEM_CLASSNAME + "__DropdownMenu");
            var mouseDownHandler = function (e) {
                e.preventDefault();
                if (!dropdown.contains(e.target)) {
                    closeDropdown();
                }
            };
            var openDropdown = function (e) {
                var target = e.target;
                if (dropdownMenu.contains(target)) {
                    return;
                }
                dropdown.classList.add(DROPWDOWN_OPEN_CLASSNAME);
                isDropdownOpen = true;
                window.addEventListener('mousedown', mouseDownHandler);
            };
            var closeDropdown = function () {
                dropdown.classList.remove(DROPWDOWN_OPEN_CLASSNAME);
                isDropdownOpen = false;
                window.removeEventListener('mousedown', mouseDownHandler);
            };
            dropdown.addEventListener('click', function (e) {
                e.preventDefault();
                if (!isDropdownOpen) {
                    openDropdown(e);
                }
                else {
                    closeDropdown();
                }
            });
            this.dropdownFields.forEach(function (dropdownItem) {
                var menuItem = menuItemsMeta[dropdownItem];
                var text = labels[menuItem.key];
                if (menuItem.key === 'heading') {
                    text += " " + menuItem.attrs.level;
                }
                var spec = {
                    classNames: [
                        DROPDWON_ITEM_CLASSNAME + "__Item"
                    ],
                    textContent: text,
                    attrs: {
                        title: text
                    },
                    activeClass: ACTIVE_DROPDOWN_ITEM_CLASSNAME,
                    disabledClass: DISABLED_CLASSNAME
                };
                var menuItemView = new MenuItemView(menuItem, _this.editorView, spec);
                var _a = menuItemView.render(), update = _a.update, dom = _a.dom;
                // remove open class once clicked on dropdown value
                dom.addEventListener('click', function (e) {
                    e.preventDefault();
                    closeDropdown();
                });
                // wrapper to execute when update is called
                var dropUpdate = function (state) {
                    update(state);
                    // update the dropdown content heading when a class is selected
                    var activeEl = dropdownMenu.getElementsByClassName(ACTIVE_DROPDOWN_ITEM_CLASSNAME);
                    if (activeEl.length) {
                        var el = activeEl[0];
                        dropdownText.textContent = el.textContent;
                        dropdown.classList.add(SELECTED_DROPDOWN_ITEM_CLASSNAME);
                    }
                    else {
                        // restore default value
                        dropdownText.textContent = labels[_this.dropdownGroup];
                        dropdown.classList.remove(SELECTED_DROPDOWN_ITEM_CLASSNAME);
                    }
                };
                dropdownMenu.appendChild(dom);
                _this.updates.push(dropUpdate);
            });
            dropdown.appendChild(dropdownMenu);
            return dropdown;
        };
        DropDownView.prototype.render = function () {
            this.dom = this.getWrapperDom();
            return {
                dom: this.dom,
                updates: this.updates
            };
        };
        return DropDownView;
    }());
    var MenuItemView = /** @class */ (function () {
        function MenuItemView(menuItem, editorView, spec) {
            this.menuItem = menuItem;
            this.editorView = editorView;
            this.spec = spec;
        }
        MenuItemView.prototype.render = function () {
            var _this = this;
            var dom = this.dom = this.getDom();
            var schema = this.editorView.state.schema;
            var command = this.setupCommandListeners().command;
            var _a = this.spec, activeClass = _a.activeClass, disabledClass = _a.disabledClass;
            var update = function (state) {
                var menuItem = _this.menuItem;
                var isActive = false;
                var canExecute = command(_this.editorView.state, null);
                if (menuItem.type === 'mark') {
                    var type = schema.marks[menuItem.key];
                    isActive = helpers.isMarkActive(state, type);
                }
                if (menuItem.type === 'node') {
                    var type = schema.nodes[menuItem.key];
                    isActive = helpers.isNodeActive(state, type, menuItem.attrs);
                }
                dom.classList.toggle(activeClass, isActive);
                dom.classList.toggle(disabledClass, !canExecute);
            };
            return {
                dom: dom,
                update: update
            };
        };
        MenuItemView.prototype.getDom = function () {
            var div = document.createElement('div');
            if (this.spec.classNames) {
                this.spec.classNames.forEach(function (className) {
                    div.classList.add(className);
                });
            }
            if (this.spec.attrs) {
                Object.entries(this.spec.attrs).forEach(function (obj) {
                    div.setAttribute(obj[0], obj[1]);
                });
            }
            if (this.spec.innerHTML) {
                div.innerHTML = this.spec.innerHTML;
            }
            if (this.spec.textContent) {
                div.innerHTML = this.spec.textContent;
            }
            return div;
        };
        MenuItemView.prototype.setupCommandListeners = function () {
            var _this = this;
            var schema = this.editorView.state.schema;
            var command;
            if (this.menuItem.type === 'mark') {
                command = prosemirrorCommands.toggleMark(schema.marks[this.menuItem.key]);
            }
            if (this.menuItem.type === 'node') {
                var type = schema.nodes[this.menuItem.key];
                if (helpers.isListItem(type, schema)) {
                    command = commands.toggleList(type, schema.nodes.list_item);
                }
                if (type === schema.nodes.heading) {
                    command = commands.toggleBlockType(type, schema.nodes.paragraph, { level: this.menuItem.attrs.level });
                }
                if (type === schema.nodes.blockquote) {
                    command = commands.toggleWrap(type);
                }
            }
            this.dom.addEventListener('mousedown', function (e) {
                e.preventDefault();
                // don't execute if not left click
                if (e.buttons !== 1) {
                    return;
                }
                // execute command
                command(_this.editorView.state, _this.editorView.dispatch);
            });
            return { command: command };
        };
        return MenuItemView;
    }());
    var getSeperatorDom = function () {
        var div = document.createElement('div');
        div.className = SEPERATOR_CLASSNAME;
        return div;
    };
    var ɵ0$1 = getSeperatorDom;
    var renderMenu = function (options, editorView, menuDom) {
        var updates = [];
        var toolbar = options.toolbar;
        toolbar.forEach(function (group, toolbarIndex) {
            var isLastMenuGroup = toolbar.length - 1 === toolbarIndex;
            group.forEach(function (toolbarItem, menuIndex) {
                var isLastMenuItem = group.length - 1 === menuIndex;
                // render dropdown
                if (typeof toolbarItem === 'object') {
                    Object.keys(toolbarItem).forEach(function (dropdownGroup) {
                        if (DROPDOWN_ITEMS.has(dropdownGroup)) {
                            var dropdown = toolbarItem[dropdownGroup];
                            var dropdownView = new DropDownView(dropdownGroup, dropdown, editorView, options);
                            var rendered = dropdownView.render();
                            updates.push(rendered.updates);
                            menuDom.appendChild(rendered.dom);
                        }
                        else {
                            console.warn('Unkown dropdown group:', dropdownGroup);
                        }
                    });
                }
                // render Icons
                if (typeof toolbarItem === 'string') {
                    var menuItem = menuItemsMeta[toolbarItem];
                    var labels = options.labels;
                    if (menuItem) {
                        var spec = {
                            classNames: [
                                MENU_ITEM_CLASSNAME,
                                MENU_ITEM_CLASSNAME + "--Icon",
                            ],
                            innerHTML: getIconSvg(menuItem.icon),
                            attrs: {
                                title: labels[menuItem.i18nKey]
                            },
                            activeClass: ACTIVE_MENU_ITEM_CLASSNAME,
                            disabledClass: DISABLED_CLASSNAME
                        };
                        var menuItemView = new MenuItemView(menuItem, editorView, spec);
                        var _a = menuItemView.render(), update = _a.update, dom = _a.dom;
                        menuDom.appendChild(dom);
                        updates.push(update);
                    }
                }
                if (typeof toolbarItem === 'function') {
                    var _b = toolbarItem(editorView), dom = _b.dom, update = _b.update;
                    menuDom.appendChild(dom);
                    updates.push(update);
                }
                if (isLastMenuItem && !isLastMenuGroup) {
                    var seperatorDom = getSeperatorDom();
                    menuDom.appendChild(seperatorDom);
                }
            });
        });
        var combinedUpdates = flatDeep(updates, Infinity);
        return {
            update: function (state) {
                combinedUpdates.forEach(function (update) {
                    update(state);
                });
            }
        };
    };

    var MenuBarView = /** @class */ (function () {
        function MenuBarView(editorView, options) {
            this.view = editorView;
            this.options = options;
            this.render();
            this.update();
        }
        MenuBarView.prototype.render = function () {
            var _a;
            var menuDom = document.createElement('div');
            menuDom.className = 'NgxEditor__MenuBar';
            var update = renderMenu(this.options, this.view, menuDom).update;
            this.updateMenuItems = update;
            (_a = this.view.dom.parentNode) === null || _a === void 0 ? void 0 : _a.insertBefore(menuDom, this.view.dom);
        };
        MenuBarView.prototype.update = function () {
            var _a;
            (_a = this.updateMenuItems) === null || _a === void 0 ? void 0 : _a.call(this, this.view.state);
        };
        return MenuBarView;
    }());

    var DEFAULT_TOOLBAR = [
        ['bold', 'italic'],
        ['code', 'blockquote'],
        ['ordered_list', 'bullet_list'],
        [{ heading: ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'] }]
    ];
    var DEFAULT_LABELS = {
        bold: 'Bold',
        italics: 'Italics',
        code: 'Code',
        ordered_list: 'Ordered List',
        bullet_list: 'Bullet List',
        heading: 'Heading',
        blockquote: 'Quote'
    };
    var DEFAULT_OPTIONS = {
        toolbar: DEFAULT_TOOLBAR,
        labels: DEFAULT_LABELS
    };
    function menuPlugin(options) {
        return new prosemirrorState.Plugin({
            key: new prosemirrorState.PluginKey('menu'),
            view: function (editorView) {
                return new MenuBarView(editorView, options);
            },
        });
    }
    var menu = function (options) {
        if (options === void 0) { options = DEFAULT_OPTIONS; }
        return menuPlugin(options);
    };
    var ɵ0$2 = menu;

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    ;
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    var FloatingOptionsView = /** @class */ (function () {
        function FloatingOptionsView(view) {
            this.bubbleEL = document.createElement('div');
            this.render(view);
            this.update(view);
        }
        FloatingOptionsView.prototype.render = function (view) {
            var _a;
            this.bubbleEL.className = 'NgxEditor__FloatingBubble';
            (_a = view.dom.parentNode) === null || _a === void 0 ? void 0 : _a.appendChild(this.bubbleEL);
        };
        FloatingOptionsView.prototype.setDomPosition = function (view) {
            // Otherwise, reposition it and update its content
            this.bubbleEL.style.display = '';
            var _b = helpers.calculateBubblePos(view, this.bubbleEL), bottom = _b.bottom, left = _b.left;
            this.bubbleEL.style.left = left + 'px';
            this.bubbleEL.style.bottom = bottom + 'px';
        };
        FloatingOptionsView.prototype.createLinkNode = function (item, removeCB) {
            var el = document.createDocumentFragment();
            var link = document.createElement('a');
            link.href = item.attrs.href;
            link.target = '_blank';
            link.innerText = item.attrs.href;
            link.title = item.attrs.href;
            var commands = document.createElement('div');
            commands.classList.add('commands');
            var editOpt = document.createElement('button');
            editOpt.classList.add('command');
            editOpt.textContent = 'Edit';
            var removeOpt = document.createElement('button');
            removeOpt.classList.add('command');
            removeOpt.textContent = 'Remove';
            removeOpt.onclick = removeCB;
            // commands.appendChild(editOpt);
            commands.appendChild(removeOpt);
            el.appendChild(link);
            el.appendChild(commands);
            return el;
        };
        FloatingOptionsView.prototype.clearBubbleContent = function () {
            this.bubbleEL.textContent = '';
        };
        FloatingOptionsView.prototype.hideBubble = function () {
            this.bubbleEL.style.display = 'none';
        };
        FloatingOptionsView.prototype.update = function (view) {
            var state = view.state, dispatch = view.dispatch;
            var selection = state.selection, schema = state.schema, doc = state.doc, tr = state.tr;
            if (!schema.marks.link) {
                return;
            }
            var hasFocus = view.hasFocus();
            var isActive = helpers.isMarkActive(state, schema.marks.link);
            var linkMarks = helpers.getSelectionMarks(state).filter(function (mark) { return mark.type === schema.marks.link; });
            // hide for selection and show only for clicks
            if (!isActive || linkMarks.length !== 1 || !hasFocus) {
                this.hideBubble();
                return;
            }
            var pos = selection.$head.pos;
            var _b = __read(linkMarks, 1), linkItem = _b[0];
            this.clearBubbleContent();
            var removeCB = function (e) {
                e.preventDefault();
                var $pos = doc.resolve(pos);
                var linkStart = pos - $pos.textOffset;
                var linkEnd = linkStart + $pos.parent.child($pos.index()).nodeSize;
                tr.removeMark(linkStart, linkEnd);
                dispatch(tr);
                view.focus();
            };
            var el = this.createLinkNode(linkItem, removeCB);
            this.bubbleEL.appendChild(el);
            // update dom position
            this.setDomPosition(view);
        };
        FloatingOptionsView.prototype.destroy = function () {
            this.bubbleEL.remove();
        };
        return FloatingOptionsView;
    }());
    function linkPlugin() {
        return new prosemirrorState.Plugin({
            key: new prosemirrorState.PluginKey('link'),
            view: function (editorView) {
                return new FloatingOptionsView(editorView);
            },
            props: {
                handleDOMEvents: {
                    blur: function (view) {
                        view.dispatch(view.state.tr.setMeta('LINK_PLUGIN_EDITOR_BLUR', true));
                        return false;
                    }
                }
            }
        });
    }

    /**
     * Generated bundle index. Do not edit.
     */

    exports.link = linkPlugin;
    exports.menu = menu;
    exports.placeholder = placeholderPlugin;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-editor-plugins.umd.js.map
